<?php
/*
Plugin Name: Flickr Widget
Plugin URI: http://localhost
Description: Этот виджет ищет по заданным тегам фотографии на сервисе flickr и показывает их.
Version: 1.0
Author: Statsenko Vladimir
Author URI: http://www.simplecoding.org
*/

//инициализация виджета, нужна для регистрации виджета и его панели управления
add_action("widgets_init", array('Flickr_Widget', 'register'));

//эти хуки не обязательные, они нужны для сохранения настроек по-умолчанию
//при активации виджета и их удаления после его отключения
register_activation_hook( __FILE__, array('Flickr_Widget', 'activate'));
register_deactivation_hook( __FILE__, array('Flickr_Widget', 'deactivate'));

/*
 Этот класс содержит основные методы для создания и управления виджетом.
 В принципе, можно использовать обычные PHP функции, но класс решает
 проблему с возможным совпадением имен.
*/
class Flickr_Widget {
    
    //название приложения (указываем при регистрации на Flickr API)
    private static $APP_NAME = 'Flickr_Widget';
    //ключ приложения (получаем при регистрации на Flickr API)
    private static $APP_KEY = '17e73e544f0f6fff4d06822668e76e01';
    
    //массив с соответствиями между суффиксами изображений и их
    //размерами (подробнее здесь - http://www.flickr.com/services/api/misc.urls.html)
    private static $IMG_SIZES = array(
                            '1'=>array('s','75 x 75 px')
                            , '2'=>array('t','max 100 px')
                            , '3'=>array('m','max 240 px')
                            , '4'=>array('','max 500 px')
                        );
    
    private static $flickr_url = 'http://api.flickr.com/services/xmlrpc/';

    /*
     Этот метод "рассказывает" движку WP какие методы создают виджет
     и его панель управления 
    */
    public function register() {
        register_sidebar_widget('Flickr Widget', array('Flickr_Widget', 'widget'));
        register_widget_control('Flickr Widget', array('Flickr_Widget', 'control'));
    }
    
    /*
     Этот метод отображает виджет.
     При этом выполняется:
     1) чтение опции fw_data
     2) поиск картинок на flickr
     3) формирование тегов img и разметки виджета
    */
    public function widget($args) {
        echo $args['before_widget'];
        //показываем заголовок виджета
        echo $args['before_title'] . 'Images from Flickr' . $args['after_title'];
        //читаем массив с опциями
        if ( $data = get_option('fw_data') ) {
            //подключаем библиотеку для работы с XML-RPC 
            include_once ('wp-includes/class-IXR.php');
            
            $client = new IXR_Client(Flickr_Widget::$flickr_url);
            $params = array('api_key'=>$data['fw_app_key']
                            ,'tags'=>$data['wf_tags']
                            ,'per_page'=>$data['wf_images']
                            );
            if ( !$client->query('flickr.photos.search', $params) ) {
                echo 'Error: '.$client->getErrorCode().':'.$client->getErrorMessage();
            }
            else {
                $photos = $client->getResponse();
                $xml = new SimpleXMLElement($photos);
                foreach ($xml->photo as $photo) {
                    $url = 'http://farm'.$photo['farm']
                            .'.static.flickr.com/'
                            .$photo['server'].'/'
                            .$photo['id'].'_'
                            .$photo['secret'];
                    $oUrl = 'http://www.flickr.com/photos/'
                            .$photo['owner']
                            .'/'.$photo['id'];
                    if ('' !== $data['fw_size']) {
                        $url .= '_'.Flickr_Widget::$IMG_SIZES[$data['fw_size']][0].'.jpg';
                    }
                    else {
                        $url .= '.jpg';
                    }
                    echo '<div class="flickr_image"><a href="'.$oUrl.'" target="_blank"><img src="'.$url.'" alt="'.$photo['title'].'" /></a></div>';
                }
            }
        }
        else {
            echo 'No settings';
        }
        echo $args['after_widget'];
    }

    /*
     Этот метод создает панель управления виджетом.
    */
    function control() {
        
        //если пришли данные в массиве $_POST, обновляем опцию
        if (isset($_POST['wf_images'])){
            $data['fw_app_name'] = Flickr_Widget::$APP_NAME;
            $data['fw_app_key'] = Flickr_Widget::$APP_KEY;
            $data['wf_images'] = esc_attr($_POST['wf_images']);
            $data['fw_size'] = esc_attr($_POST['fw_size']);
            $data['wf_tags'] = esc_attr($_POST['wf_tags']);
            update_option('fw_data', $data);
        }
        
        //читаем данные, сохраненные в опции
        $data = get_option('fw_data');
        if ( FALSE === $data ) {
            //заполняем массив $data значениями по-умолчанию
            $data = array();
            $data['wf_images'] = '';
            $data['fw_size'] = '';
            $data['wf_tags'] = '';
        }
        //создаем поля формы, которая будет показана в панели управления
?>
        <p><label>Количество картинок <input name="wf_images"
        type="text" value="<?php echo $data['wf_images']; ?>" /></label></p>
        <p><label>Размер картинок <select size="1" name="fw_size">
<?php
        foreach ( Flickr_Widget::$IMG_SIZES as $i => $size ) {
            $selected = '';
            if ( $i == $data['fw_size'] ) {
                $selected = 'selected="selected"';
            }
?>
            <option value="<?php echo $i; ?>" <?php echo $selected; ?>><?php echo $size['1']; ?></option>
<?php
        }
?>
        </select></label></p>
        <p><label>Теги (через запятую) <input name="wf_tags"
        type="text" value="<?php echo $data['wf_tags']; ?>" /></label></p>
<?php
    }

    /*
     В БД будет сохранена опция с названием fw_data,
     а ее значением будет массив с данными
     (WordPress сериализует их автоматически).
     Этот метод вызывается автоматически при активации виджета.
     */
    function activate() {
        //устанавливаем начальные значения
        $data['fw_app_name'] = Flickr_Widget::$APP_NAME;
        $data['fw_app_key'] = Flickr_Widget::$APP_KEY;
        $data['wf_images'] = '2'; //две картинки
        $data['fw_size'] = '2'; //размер не более 100px по большей стороне
        $data['wf_tags'] = 'nature, природа';
        //и создаем опцию
        add_option('fw_data', $data);
    }
    
    /*
     Удаляет опцию fw_data из базы при отключении виджета.
     Этот метод вызывается автоматически при отключении виджета.
    */
    function deactivate() {
        //удаляем опцию
        delete_option('fw_data');
    }
    
}

//end of flickr-widget.php